package TreeStore;
#
# $Id: TreeStore.pm,v 1.4 2002/10/24 03:07:11 white Exp $
#
use strict;
use vars qw(@ISA $AUTOLOAD);

use TreeStore::File;
use TreeStore::Dir;
use TreeStore::Undef;


my(%cache);
my($BaseDir);
my($CREATE_MODE_DIR)  = 0777;
my($CREATE_MODE_FILE) = 0666;

sub import{
}

sub new{
    my $self  = shift;
    my $class = ref($self) || $self;
    my($base) = bless { BASE => 1 } , $class;
    $self = {path => "" , base => $base} if($class eq $self);
    my($name) = @_;
    return $base->get($self,$name);
}

sub get{
    my($parent) = shift;
    my($base);
    if($parent->{BASE}){
	$base = $parent;
	$parent = shift;
    }
    my($target) = @_;
    my($path) = $base->join_path($parent->{path},$target);
    my($name) = $base->join_path($parent->{name},$target);
    my($child) = { path => $path, name => $name , base => $parent->{base} };
    if(-d $path){
	return bless $child, "TreeStore::Dir";
    }elsif(-f _){
	return bless $child, "TreeStore::File";
    }else{
	return bless $child, "TreeStore::Undef";
    }
}

sub parent{
}

sub delete{
}

sub read_dir{
    (my $self,my $path,@_) = &__PARSEARG(@_);
    my @result = $self->read_dir_cache($path);
    if(wantarray){
        return @result;
    }else{
        return join("\n",@result);
    }
}

sub read_file{
    (my $self,my $path,@_) = &__PARSEARG(@_);
    my @result = $self->read_file_cache($path);
    if(wantarray){
        return @result;
    }else{
        return join("\n",@result);
    }
}

sub read_file_cache{
    (my $self,my $path) = @_;
    if(not defined $self->{cache_read}->{$path}){
	open(READ,"< $path");
	my @read = <READ>;
	close(READ);
	chomp(@read);
	$self->{cache_read}->{$path} = \@read;
    }
    return @{$self->{cache_read}->{$path}};
}

sub read_dir_cache{
    (my $self,my $path) = @_;
    if(not defined $self->{cache_read}->{$path}){
	opendir(DIR,"$path");
	my(@read)= readdir(DIR);
	closedir(DIR);
	return sort @read;
	$self->{cache_read}->{$path} = \@read;
    }
    return @{$self->{cache_read}->{$path}};
}

sub write_file{
    (my $self,my $path,@_) = &__PARSEARG(@_);
    my($content) = shift;
    open(WRITE,"> $path");
    print WRITE $content;
    close(WRITE);
    my @tmp = split(/\n/,$content);
    $self->{cache_read}->{$path} = \@tmp;
}

sub join_path{
    (my $self,@_) = &__PARSEARG(@_);
    local($_);
    while(@_){
	$_ = shift(@_);
	last if($_ !~ /^\/*$/);
    }
    my $path = $_;
    foreach(@_){
	next if($_ =~ /^\/*$/);
	if($path =~ /\/$/){
	    $path .= $_;
	}else{
	    $path .= "/$_";
	}
    }
    return $path;
}

sub __PARSEARG{
    my($path) = shift;
    my($self);
    if(ref($path)){
	$self = $path;
	$path = shift;
	$path = $path->{path} if ref($path);
    }
    return ($self,$path,@_);
}

sub DESTROY{
#    Future plan : write here flushing routine works when set() executed.
    my($self) = shift;
    if($self->{value_seted} == 1){
	$self->flush;
    }
}
1;
